@@ -12,7 +12,9 @@ class Zot:
 
     def __init__(self, library_id, api_key):
         """All of our interactions here require a Zot instance that connects to a specific library ID
-        with an API key that supports read/write access.
+        with an API key that supports read/write access. Initiating a Zot instance establishes the Zotero
+        API connection (self.z), passes on the library_id, and pulls up connection information used to
+        determine if the connection has write access in several other functions (self.cnxn).
 
         Args:
             library_id (str): Alphanumeric identifier for a particular library
@@ -29,7 +31,38 @@ class Zot:
         except Exception as e:
             raise ValueError(f"Problem creating connection to Zotero API: {e}")
 
-    def get_inventory_files(self, inventory_item):
+    def inventory_item_key(self):
+        """If necessary, this function can be used to identify the item containing the inventory cache in the library
+        using a specific convention on the item's title - "Inventory:<library_id>".
+
+        Returns:
+            str: Key (identifier) of the library item providing the cached inventory of items, collections, and tags.
+        """
+        inventory_item_title = f"Inventory:{self.library_id}"
+        inventory_items = self.z.items(q=inventory_item_title, itemType="document")
+        inventory_item = next((i for i in inventory_items if i["data"]["title"] == inventory_item_title), None)
+        if inventory_item is None:
+            return
+
+        return str(inventory_item["key"])
+
+    def get_inventory_files(self, inventory_item=None):
+        """To facilitate efficient access to larger libraries, we established a method of building and maintaining
+        a specific item in the library where we cache inventory data as JSON files which can be retrieved in lieu
+        of what can be a time-consuming pull of the entire recordset via the API. If the inventory item is not
+        explicitly provided, the inventory_item_key() function is used to try and find it.
+
+        Args:
+            inventory_item (str): Alphanumeric identifier for the inventory item in a given library. Defaults to None.
+
+        Returns:
+            list: List of dictionaries from the Zotero API containing the inventory item and its files
+        """
+        if inventory_item is None:
+            inventory_item = self.inventory_item_key()
+            if inventory_item is None:
+                return
+        
         inventory_files = self.z.children(inventory_item, itemType="attachment")
         inventory_file_items = {
             "inventory_item": self.z.item(inventory_item),
@@ -45,28 +78,51 @@ class Zot:
         return inventory_file_items
 
     def load_raw_inventory(self, inventory_item):
-        existing_files = self.get_inventory_files(inventory_item)
+        """This function loads the raw inventory information for items, collections, and tags stored in the inventory
+        item as JSON files.
+
+        Args:
+            inventory_item (str): Alphanumeric identifier for the inventory item in a given library
+
+        Returns:
+            dict: Dictionary containing lists for each set of cached records - items, collections, and tags
+        """
+        inventory_files = self.get_inventory_files(inventory_item)
+        if inventory_files is None:
+            raise ValueError("The inventory item could not be found in the library.")
 
         inventory = {
-            "inventory_item_keys": existing_files["inventory_item_keys"],
-            "inventory_files": existing_files
+            "inventory_item_keys": inventory_files["inventory_item_keys"],
+            "inventory_files": inventory_files
         }
         for file_type in ["items","collections","tags"]:
-            raw_data = self.z.file(existing_files[file_type]["key"])
+            raw_data = self.z.file(inventory_files[file_type]["key"])
             inventory[file_type] = raw_data
         
         return inventory
 
-    def load_df_inventory(self, raw_inventory):
+    def load_df_inventory(self, raw_inventory, tag_delimiter=":"):
+        """This function takes the raw inventory dict containing three sets of records for items, collections,
+        and tags and builds Pandas dataframes for further processing. We do some minimal processing on tags
+        to split these out using a ":" delimiter by default.
+
+        Args:
+            raw_inventory (dict): Dictionary contanining three keys with lists for items, collections, and tags
+            from load_raw_inventory()
+            tag_delimiter (str, optional): Delimiter to use in splitting tags into classes. Defaults to ":".
+
+        Returns:
+            dict: Dictionary containing three dataframes for items, collections, and tags
+        """
         inventory = {}
         for k,v in raw_inventory.items():
             if k == "tags":
                 df_tags = pd.DataFrame([
                     {
                         "tag": i,
-                        "type": i.split(":")[0],
-                        "value": i.split(":")[-1]
-                    } for i in v if ":" in i
+                        "type": i.split(tag_delimiter)[0],
+                        "value": i.split(tag_delimiter)[-1]
+                    } for i in v if tag_delimiter in i
                 ])
                 inventory["tags"] = df_tags.convert_dtypes()
             elif k in ["items","collections"]:
@@ -75,17 +131,37 @@ class Zot:
         return inventory
 
     def baseline_tags(self, output_path="data"):
+        """This function gets all tags for a library and dumps them as a JSON file to a specified output path.
+
+        Args:
+            output_path (str, optional): Relative or absolute path to output json file. Defaults to "data".
+        """
         tags = self.z.everything(self.z.tags())
         json.dump(tags, open(f"{output_path}/tags.json", "w"))
 
     def baseline_collections(self, output_path="data"):
+        """This function gets all collections for a library and dumps them as a JSON file to a specified output path.
+
+        Args:
+            output_path (str, optional): Relative or absolute path to output json file. Defaults to "data".
+        """
         records = self.z.everything(self.z.collections())
         json.dump(records, open(f"{output_path}/collections.json", "w"))
 
-    def baseline_items(self, inventory_item, output_path="data"):
+    def baseline_items(self, inventory_item=None, output_path="data"):
+        """This function gets all items (metadata and files) for a library and dumps them as a JSON file to a 
+        specified output path. It will strip out any items associated with the inventory item.
+
+        Args:
+            inventory_item (str): Alphanumeric identifier for the inventory item in a given library
+            output_path (str, optional): Relative or absolute path to output json file. Defaults to "data".
+        """
         records = self.z.everything(self.z.items())
         
         inventory_files = self.get_inventory_files(inventory_item)
+        if inventory_files is None:
+            raise ValueError("The inventory item could not be found in the library.")
+
         records = [
             i for i in records 
             if i["key"] not in inventory_files["inventory_item_keys"]
@@ -94,13 +170,26 @@ class Zot:
 
     def baseline_cache(
         self, 
-        inventory_item, 
+        inventory_item=None, 
         output_path="data"
     ):
+        """This function takes the files found in a specified output_path (from baseline or update) for
+        items, collections, and tags and uploads these to the specified inventory item in the library.
+        This function can only be successfully operated using an API key with write permissions.
+
+        Args:
+            inventory_item (str): Alphanumeric identifier for the inventory item in a given library
+            output_path (str, optional): Relative or absolute path to output json file. Defaults to "data".
+
+        Returns:
+            bool: False if the API key does not have permission to write to the library
+        """
         if not self.cnxn["access"]["user"]["write"]:
             return False
 
         inventory_files = self.get_inventory_files(inventory_item)
+        if inventory_files is None:
+            raise ValueError("The inventory item could not be found in the library.")
 
         new_uploads = []
         for file_type in ["items","collections","tags"]:
@@ -122,11 +211,26 @@ class Zot:
             self.z.attachment_simple(new_uploads, parentid=inventory_item)
             print("Created new files in inventory", new_uploads)
 
-    def update_inventory(self, inventory_item, output_path="data"):
+    def update_inventory(self, inventory_item=None, output_path="data"):
+        """This function handles a periodic refresh of the cached inventory. It reads the current inventory
+        files, determines the last version of the library cached, retrieves new records (items, collections, tags),
+        and updates the cached files. This function can only be successfully operated using an API key with write
+        permissions.
+
+        Args:
+            inventory_item (str): Alphanumeric identifier for the inventory item in a given library
+            output_path (str, optional): Relative or absolute path to output json file. Defaults to "data".
+
+        Returns:
+            bool: False if the API key does not have permission to write to the library
+        """
         if not self.cnxn["access"]["user"]["write"]:
             return False
 
         inventory_files = self.get_inventory_files(inventory_item)
+        if inventory_files is None:
+            raise ValueError("The inventory item could not be found in the library.")
+
         raw_inventory = self.load_raw_inventory(inventory_item)
 
         cache_item_version = max([i["version"] for i in raw_inventory["items"]])
@@ -200,6 +304,14 @@ class Zot:
             print("Created new files in inventory", new_uploads)
             
     def build_identifier(self, record):
+        """Builds identifier objects for a particular type of export format.
+
+        Args:
+            record (series): Dataframe series (record)
+
+        Returns:
+            list: List of identifier objects/dicts
+        """
         identifiers = []
         identifiers.append({
             "type": "Zotero Item Key",
@@ -212,6 +324,14 @@ class Zot:
         return identifiers
 
     def build_link(self, record):
+        """Builds link objects for a particular type of export format.
+
+        Args:
+            record (series): Dataframe series (record)
+
+        Returns:
+            list: List of link objects/dicts
+        """
         items_api = f"https://api.zotero.org/groups/{self.library_id}/items"
         items_ui = f"https://www.zotero.org/groups/{self.library_id}/items"
 
@@ -231,6 +351,17 @@ class Zot:
         return links
 
     def collection_tags(self, geo_tags=True, custom_tags=None):
+        """This is a fairly specialized function that interprets collections from the Zotero library to turn these
+        into additional type-classified tags. 
+
+        Args:
+            geo_tags (bool, optional): Specifies whether or not to check collections for geo names. Defaults to True.
+            custom_tags (list, optional): List of lists of specific types of additional tags that should be found within
+            the collection structure of a library. Defaults to None.
+
+        Returns:
+            list: List of additional tags as compound, type-classified strings containing a type and term.
+        """
         raw_collections = self.z.everything(self.z.collections())
 
         collection_tags = []
@@ -261,8 +392,24 @@ class Zot:
 
         return collection_tags
 
-    def item_export(self, inventory_item, target_schema="xdd", collection_tag_mapping=None, output_format="dataframe"):
-        raw_inventory = self.load_raw_inventory(inventory_item)
+    def item_export(self, inventory_item=None, target_schema="xdd", output_format="dataframe"):
+        """This function packages the items for a given library using the cached inventory files and returns them for
+        some particular use. We need to better work up the configuration details for this so that we can accept some
+        form of configuration principals that are used to determine the appropriate mappings or transformations.
+
+        Args:
+            inventory_item (str): Alphanumeric identifier for the inventory item in a given library
+            target_schema (str, optional): The particular target output being transformed to. Defaults to "xdd".
+            output_format (str, optional): Determines the output format for the transformation. Defaults to "dataframe".
+
+        Returns:
+            dataframe/list: Either a dataframe (default) containing the transformation or a list of dicts.
+        """
+        try:
+            raw_inventory = self.load_raw_inventory(inventory_item)
+        except ValueError as e:
+            raise ValueError(e)
+
         df_inventory = self.load_df_inventory(raw_inventory)
 
         items = df_inventory["items"]
